/***************************************************************************
 *   Copyright (C) 2005-2007 Nicolas Hadacek <hadacek@kde.org>             *
 *   Copyright (C) 2004 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "config_center.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qtimer.h>
#include <kiconloader.h>
#include <klocale.h>
#include <klistview.h>

#include "global_config.h"
#include "device_gui.h"
#include "tools/list/tools_config_widget.h"
#include "progs/gui/prog_config_center.h"
#include "tools/list/compile_config.h"

//----------------------------------------------------------------------------
GeneralConfigWidget::GeneralConfigWidget()
  : GridConfigWidget(0, "general_config_widget")
{
  uint row = 0;

  _autoRebuildModified = new QCheckBox(i18n("Automatically rebuild project before programming if it is modified."), this);
  _topLayout->addMultiCellWidget(_autoRebuildModified, row,row, 0,1);
  row++;

  _programAfterBuild = new QCheckBox(i18n("Program device after successful build."), this);
  _topLayout->addMultiCellWidget(_programAfterBuild, row,row, 0,1);
  row++;

  _showDebug = new QComboBox(this);
  _showDebug->insertItem(i18n(Log::DEBUG_DATA[Log::Nb_DebugLevels].label));
  for (uint i=0; i<Log::Nb_DebugLevels; i++)
    _showDebug->insertItem(i18n(Log::DEBUG_DATA[i].label));
  _topLayout->addWidget(_showDebug, row, 0);
  row++;

  _userIdSetToChecksum = new QCheckBox(i18n("Set User Ids to unprotected checksum (if User Ids are empty)."), this);
  _topLayout->addMultiCellWidget(_userIdSetToChecksum, row,row, 0,1);
  row++;

  _showTabCloseButtons = new QCheckBox(i18n("Show close buttons on tabs (need restart to take effect)."), this);
  _topLayout->addMultiCellWidget(_showTabCloseButtons, row,row, 0,1);
  row++;
}

void GeneralConfigWidget::loadConfig()
{
  GlobalConfig config;
  _autoRebuildModified->setChecked(config.autoRebuildModified());
  _programAfterBuild->setChecked(config.programAfterBuild());
  Log::DebugLevel level = config.debugLevel();
  if ( level!=Log::Nb_DebugLevels ) _showDebug->setCurrentItem(level+1);
  _userIdSetToChecksum->setChecked(config.isUserIdSetToChecksum());
  _showTabCloseButtons->setChecked(config.showTabCloseButtons());
}

void GeneralConfigWidget::saveConfig()
{
  GlobalConfig config;
  config.writeAutoRebuildModified(_autoRebuildModified->isChecked());
  config.writeProgramAfterBuild(_programAfterBuild->isChecked());
  uint i = _showDebug->currentItem();
  config.writeDebugLevel(i==0 ? Log::Nb_DebugLevels : Log::DebugLevel(i-1));
  config.writeUserIdSetToChecksum(_userIdSetToChecksum->isChecked());
  config.writeShowTabCloseButtons(_showTabCloseButtons->isChecked());
}

QPixmap GeneralConfigWidget::pixmap() const
{
  KIconLoader loader;
  return loader.loadIcon("configure", KIcon::Toolbar, KIcon::SizeMedium);
}

//----------------------------------------------------------------------------
StandaloneConfigWidget::StandaloneConfigWidget()
  : ConfigWidget(0, "standalone_config_widget")
{
  QGridLayout *top = new QGridLayout(this, 1, 1, 10, 10);

  QLabel *label = new QLabel(i18n("Device:"), this);
  top->addWidget(label, 0, 0);
  _device = new DeviceChooser::Button(true, this);
  top->addWidget(_device, 0, 1);

  _tools = new ToolsConfigWidget(0, this);
  top->addMultiCellWidget(_tools, 1,1, 0,2);

  top->setRowStretch(1, 1);
  top->setColStretch(2, 1);
}

void StandaloneConfigWidget::loadConfig()
{
  _device->setDevice(Compile::Config::device(0));
  _tools->loadConfig();
}

void StandaloneConfigWidget::saveConfig()
{
  Compile::Config::setDevice(0, _device->device());
  _tools->saveConfig();
}

QPixmap StandaloneConfigWidget::pixmap() const
{
  KIconLoader loader;
  return loader.loadIcon("configure", KIcon::Toolbar, KIcon::SizeMedium);
}

//----------------------------------------------------------------------------
ConfigWidget *ConfigCenter::factory(Type type)
{
  switch (type) {
    case General:     return new GeneralConfigWidget;
    case ProgSelect:  return new Programmer::SelectConfigWidget;
    case ProgOptions: return new Programmer::OptionsConfigWidget;
    case Standalone:  return new StandaloneConfigWidget;
    case Nb_Types:    break;
  }
  Q_ASSERT(false);
  return 0;
}

ConfigCenter::ConfigCenter(Type showType, QWidget *parent)
  : Dialog(IconList, i18n("Configure Piklab"), Ok|Cancel, Cancel,
           parent, "configure_piklab_dialog", true, false)
{
  for (uint i=0; i<Nb_Types; i++) {
    _configWidgets[i] = factory(Type(i));
    _configWidgets[i]->loadConfig();
    _pages[i] = addPage(_configWidgets[i]->title(), _configWidgets[i]->header(), _configWidgets[i]->pixmap());
    QVBoxLayout *vbox = new QVBoxLayout(_pages[i]);
    _configWidgets[i]->reparent(_pages[i], QPoint(0,0));
    vbox->addWidget(_configWidgets[i]);
  }
  showPage(showType);
}

void ConfigCenter::slotApply()
{
  for (uint i=0; i<Nb_Types; i++)
    _configWidgets[i]->saveConfig();
}

void ConfigCenter::slotOk()
{
  slotApply();
  accept();
}
